#ifndef lint
static char sccs_id[] = "%W%	(TriChlor) %G% %U%";
#endif  /* lint */


#include <stdio.h>
#include <vntype.h>


#define u_char			unsigned char

#ifndef DEFAULT_PSLIBDIR
#define	DEFAULT_PSLIBDIR	"/usr/local/lib/ps"
#endif

/* environment variable */
#define PSLIBDIR		"PSLIBDIR"


char pslibdir[200]	=	DEFAULT_PSLIBDIR;


Usage()
{
    fprintf(stderr, "Usage: vnpstext [-i prolog_file] [input file(s)...]\n");
    fprintf(stderr, "PostScript output will go to stdout.\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "Purpose: converts 8-bit VISCII file to printable PostScript\n");
    fprintf(stderr, "Version: VISCII 1.1\n");
    fprintf(stderr, "Author:  Cuong T. Nguyen (cuong@haydn.Stanford.EDU\n");
    fprintf(stderr, "         TriChlor Software Group---Copyrighted Freeware\n");
    exit(1);
}


main(ac, av)
    int		ac;
    char	*av[];
{
    int		index = 1;
    FILE	*input;

    if (av[1] && !strcmp(av[1], "-i")) {
	/* Prolog file is specified */
	if (av[2]) {
	    doprolog(av[2]);
	} else {
	    fprintf(stderr, "%s: -i requires prolog file name\n", av[0]);
	    Usage();
	}
	index = 3;
    } else {
	if (av[1] && av[1][0] == '-') Usage();
	doprolog(0); /* use built-in prolog file */
    }
    if (index >= ac) { /* read stdin */
	dofile();
    } else { /* filenames specified */
	FILE	*result;

	for (; index < ac; index++) {
	    result = freopen(av[index], "r", stdin);
	    if (result == NULL) {
		fprintf(stderr, "%s: can't open file \"%s\"\n", av[0], av[index]);
		Usage();
	    }
	    dofile();
	}
    }
}


/*
 * dofile: reads from stdin and converts to PS, writes to stdout.
 *
 *	line		->	(line) S
 *	empty line	->	B
 *	n empty lines	->	n L
 *	new page	->	%%Page n n	# used only in PS
 *	start page	->	StartPage	# not used
 *	end page	->	EndPage		# not used
 *	non-ascii char	->	octal code
 *	65 lines per page			# determined in PS
 *
 */
dofile(input)
    FILE		*input;
{
    u_char		line[1024];
    register u_char	*s;
    register int	pageno = 1;
    register short	col = 0;
    short		i;


    printf("StartDoc\n");

    while (gets(line)) {
	if (!*line) printf("B\n");
	else {

	    putchar('(');

	    for (s = line, col = 0; *s; s++, col++) {
		if (*s < 0x80) {
		    if (*s < 0x20 && isalpha(*s)) { /* A Viet char in  C0 */
			/* output octal code */
			printf("\\%03o", *s);
			continue;
		    }
		    switch(*s) {
		    case '(':
		    case ')':
		    case '\\':	/* must quote these guys */
			printf("\\%c", *s);
			break;
		    case '\t':	/* simulate a tab */
			/* move to next greater multiple of 8 */
			i = 8 * ((col + 8) / 8);
			for (; col < i; col++) putchar(' ');
			col--; /* to be incremented in the for() */
			break;
		    case '\b':	/* simulate a backspace */
			/* close up the current string, */
			/* start a new string at one column to the left */
			printf(")\n(");	/* close up the current string */

			/* if at anything greater than column 1, */
			/* issue col-1 spaces */
			if (col) col--;
			for (i = 0; i < col; i++) putchar(' ');

			/* back up one more column to nullify effect of col++ */
			col--;
			break;
		    case '\f':	/* form-feed */
			/* close up the current string */
			printf(")S\n");
			putchar('(');
			break;
		    default: /* output as is if ASCII */
			putchar(*s);
			break;
		    }
		}
		/* else output octal code */
		else printf("\\%03o", *s);
	    }

	    printf(")S\n");

	}
    }

    printf("showpage\n");
    printf("EndDoc\n");
}


doprolog(file)
    char	*file;
{
    char	*dir, *getenv();
    int		fd, n;
    char	buf[1024];

    if (!file || !*file) { /* no file, use built-in */
	printf("%%! 10-point VN-Courier prolog file\n");
	printf("/size 10 def\n");
	printf("/lines 65 def\n");
	printf("/Courier-VN findfont size scalefont setfont\n");
	printf("/StartDoc{/sv save def 48 760 moveto} def\n");
	printf("/EndDoc{sv restore} def\n");
	printf("/Curline 0 def\n");
	printf("/Doit {EndPage StartPage} def\n");
	printf("/Lcheck { /Curline Curline 1 add def\n");
	printf("Curline lines eq { /Curline 0 def showpage 48 760 moveto } if\n");
	printf("} def\n");
	printf("/ld size -1.036 mul def\n");
	printf("/S{Lcheck count{gsave show grestore}repeat 0 ld rmoveto}def\n");
	printf("/L{Lcheck ld mul 0 exch rmoveto}def\n");
	printf("/B{Lcheck 0 ld rmoveto}def\n");
	return;
    }

    /* filename specified, use it */
    if ( (fd = open(file, 0)) < 0 ) { /* not there, relative to PSLIBDIR? */
	if (dir = getenv(PSLIBDIR)) {
	    strcpy(pslibdir, dir);
	}
	strcat(pslibdir, "/");
	strcat(pslibdir, file);
	if ( (fd = open(pslibdir, 0)) < 0 ) {
	    perror(file);
	    exit(1);
	}
    }

    while ( (n = read(fd, buf, sizeof(buf))) > 0 ) {
	write(1, buf, n);
    }
}
